<?php

declare(strict_types=1);

namespace app\controllers;

use app\models\Notification;
use yii;
use yii\filters\AccessControl;
use yii\web\Controller;
use yii\web\Response;

use app\models\Order;
use app\models\OrderEdgeSupply;

use Exception;

class CartController extends Controller
{
    /**
     * Страница: "Корзина"
     *
     * @see $this->behaviors Доступ только аутентифицированным
     */
    public function actionIndex(): string|array|null
    {
        // Инициализация
        $page = yii::$app->request->get('page') ?? yii::$app->request->post('page') ?? 1;
        $account = yii::$app->user->identity;

        // Поиск корзины (текущего заказа)
        $model = Order::search();

        if (empty($model)) {
            // Корзина не инициализирована

            // Инициализация
            $model = new Order();
            $model->save() or throw new Exception('Не удалось инициализировать заказ');

            // Подключение
            $model->connect($account);
        }

        // Инициализация содержимого корзины
        $connections = $model->content(10, $page);

        if (yii::$app->request->isPost) {
            // POST-запрос

            yii::$app->response->format = Response::FORMAT_JSON;

            return [
                'main' => $this->renderPartial('index', compact('model', 'connections')),
                'title' => 'Корзина',
                'redirect' => '/cart',
                '_csrf' => yii::$app->request->getCsrfToken()
            ];
        }

        return $this->render('index', compact('model', 'connections'));
    }

    public function actionEditComm(string $catn): array|string|null
    {
        // Инициализация
        $return = [
            '_csrf' => yii::$app->request->getCsrfToken()
        ];

        if (is_null($catn)) {
            // Не получен артикул

            yii::$app->response->statusCode = 500;

            goto end;
        }

        if ($edges = OrderEdgeSupply::searchBySupplyCatn($catn, Order::search())) {
            // Рёбра найдены (связи заказа с поставкой)

            // Инициализация
            $amount = 0;

            foreach ($edges as $edge) {
                // Перебор рёбер (связей заказа с поставкой)

                // Инициализация
                $text = yii::$app->request->post('text') ?? yii::$app->request->get('text') ?? 'Комментарий к заказу';

                $comm = $edge->comm ?? null;
                $edge->comm = $text;

                if ($edge->save()) {
                    // Ребро обновлено

                    // Запись в журнал
                    $edge->journal('update', ['comm' => ['from' => $comm, 'to' => $edge->comm]]);

                    // Обновление счётчика
                    ++$amount;

                    // Запись в буфер ответа
                    $return['comm'] = $text;
                }
            }

            if ($amount > 0) {
                // Удалось записать минимум 1 связь с поставкой

                Notification::_write("Обновлён комментарий к товару $catn ($amount шт)");
            } else {
                // Не удалось записать минимум 1 связь с поставкой

                Notification::_write("Не удалось обновить комментарий к товару $catn");
            }
        }

        /**
         * Конец алгоритма
         */
        end:

        if (yii::$app->request->isPost) {
            // POST-запрос

            yii::$app->response->format = Response::FORMAT_JSON;

            return $return;
        }

        if ($model = Product::searchByCatn($catn)) {
            return $this->render('index', compact('model'));
        } else {
            return $this->redirect('/');
        }
    }
}
