<?php

declare(strict_types=1);

namespace app\models;

class City extends Document
{
    public static function collectionName(): string
    {
        return 'city';
    }

    public function attributes(): array
    {
        return array_merge(
            parent::attributes(),
            [
                'indx',
                'name',
                'code',
                'term'
            ]
        );
    }

    public function rules(): array
    {
        return array_merge(
            parent::rules(),
            [
                [
                    'indx',
                    'string'
                ],
                [
                    [
                        'indx',
                        'name',
                        'code',
                        'term'
                    ],
                    'required',
                    'message' => 'Заполните поле: {attribute}'
                ],
                [
                    [
                        'name',
                        'code'
                    ],
                    'string'
                ],
                [
                    'term',
                    'boolean'
                ]
            ]
        );
    }

    public function attributeLabels(): array
    {
        return array_merge(
            parent::attributeLabels(),
            [
                'indx' => 'Идентификатор в ДеловыеЛинии',
                'name' => 'Название',
                'code' => 'Код КЛАДР',
                'term' => 'Указатель наличия терминала в городе'
            ]
        );
    }

    /**
     * Поиск по идентификатору в ДеловыеЛинии
     */
    public static function searchByDellinId(string $indx): ?static
    {
        return static::findOne(['indx' => $indx]);
    }
}
