/**
 * Записать в корзину (создать корзину, если не существует)
 */
function cart_write(catn, amount = 1) {
    // Инициализация
    let data = {};
    data[catn] = amount;

    $.ajax({
        url: '/order/write',
        type: 'post',
        dataType: 'json',
        data: {
            '_csrf': yii.getCsrfToken(),
            'supplies': data
        },
        success: cart_response_success,
        error: cart_response_error
    });

    return false;
}

/**
 * Удалить корзину
 */
function cart_delete() {
    $.ajax({
        url: '/order/delete',
        type: 'post',
        dataType: 'json',
        data: {
            '_csrf': yii.getCsrfToken()
        },
        success: cart_response_success,
        error: cart_response_error
    });

    return false;
}

/**
 * Сформировать заказ
 */
function cart_pay() {
    $.ajax({
        url: '/order/pay',
        type: 'post',
        dataType: 'json',
        data: {
            '_csrf': yii.getCsrfToken()
        },
        success: cart_response_success,
        error: cart_response_error
    });

    return false;
}

/**
 * Управление чекбоксами
 */
function cart_list_checkbox(target) {
    // Инициализация
    let elements = document.getElementsByClassName('cart_list_target');
    let reg = /^\w+_([^_]*)$/;
    let catn = reg.exec(target.id)[1];

    if (catn === 'all') {
        if (target.checked === true) {
            for ($i = 0; $i < elements.length; $i++) {
                // Перебор всех выбранных элементов

                elements[$i].getElementsByTagName('input')[0].checked = true;
            }

            target.checked = true;
        } else {
            for ($i = 0; $i < elements.length; $i++) {
                // Перебор всех выбранных элементов

                elements[$i].getElementsByTagName('input')[0].checked = false;
            }

            target.checked = false;
        }

        return false;
    }
}

/**
 * Удалить из корзины
 */
function cart_list_delete(target, amount = 0) {
    // Инициализация
    let targets = {};

    if (target !== undefined) {
        // Обработка входных параметров

        targets[target] = amount;
    } else {
        // Обработка выбранных элементов в списке

        // Инициализация
        let elements = document.getElementsByClassName('cart_list_target');
        let reg = /^\w+_([^_]*)$/;

        for ($i = 0; $i < elements.length; $i++) {
            // Перебор всех выбранных элементов

            // Инициализация
            let checkbox = elements[$i].getElementsByTagName('input')[0];

            if (checkbox.checked === true) {
                targets[reg.exec(checkbox.id)[1]] = 0;
            }
        }
    }

    $.ajax({
        url: '/order/delete',
        type: 'post',
        dataType: 'json',
        data: {
            '_csrf': yii.getCsrfToken(),
            'targets': targets
        },
        success: cart_response_success,
        error: cart_response_error
    });

    // Реинициализация
    document.getElementById('cart_list_action').value = 'none';

    // Пересчитывание стоимости
    cart_cost_calculate();

    return false;
}

/**
 * Изменить количество товара в корзине
 */
function cart_list_amount_update(target, input) {
    if (target !== undefined && input !== undefined) {
        // Обработка входных параметров

        // Инициализация
        let targets = {};
        targets[target] = input.value;

        $.ajax({
            url: '/order/amount-update',
            type: 'post',
            dataType: 'json',
            data: {
                '_csrf': yii.getCsrfToken(),
                'targets': targets
            },
            success: cart_response_success,
            error: cart_response_error
        });
    }

    // Пересчитывание стоимости
    cart_cost_calculate();

    return false;
}

/**
 * Подсчёт стоимости
 */
function cart_cost_calculate() {
    let elements = document.getElementsByClassName('cart_list_target');
    let reg = /^([0-9]*)\s*\w*/;
    let costs = 0;

    for ($i = 0; $i < elements.length; $i++) {
        let cost = elements[$i].getElementsByTagName('div')[5].innerText;
        let amount = elements[$i].getElementsByTagName('div')[3].children[0].value;

        costs += +reg.exec(cost)[1] * amount;
    }

    document.getElementById('cart_cost').innerHTML = costs;
}

/**
 * Изменить количество товара в корзине
 */
 function cart_list_comment_edit(catn, element) {
    if (catn !== null && catn !== undefined && element !== null && element !== undefined) {
        element.innerHTML = '<textarea class="form-control" cols="50" rows="5" onchange = "return cart_list_comment_save(\'' + catn + '\', this.parentElement)">' + element.innerText + '</textarea>';

        element.removeAttribute('onclick');

        return false;
    }

    return true;
}

function cart_list_comment_save(catn, element) {
    if (catn !== null && catn !== undefined && element !== null && element !== undefined) {
        // Инициализация
        let text = element.children[0].value;

        // Обновление заголовка (предзагрузка)
        element.innerHTML = text;

        // Запись аттрибута (предзагрузка)
        element.setAttribute('onclick', 'return cart_list_comment_edit(\'' + catn + '\', this);');

        $.ajax({
            url: '/cart/' + catn + '/edit/comm',
            type: 'post',
            dataType: 'json',
            data: {
                '_csrf': yii.getCsrfToken(),
                'text': text
            },
            success: function (data, status) {
                // Заголовок
                if (data.comm !== undefined && element !== null && element !== undefined) {
                    // Обновление заголовка
                    element.innerHTML = data.comm;

                    // Запись аттрибута
                    element.setAttribute('onclick', 'return cart_list_comment_edit(\'' + catn + '\', this);');
                };

                cart_response_success(data, status);
            },
            error: cart_response_error
        });

        return false;
    }

    return true;
}

cart_cost_calculate();

function cart_response(data, status) {
    // Обработка ответов

    // Основной блок
    if (data.main !== undefined) {
        main = document.getElementsByTagName('main')[0];

        // Обновление документа
        main.innerHTML = data.main;

        // Реинициализация
        reinitialization(main);
    }

    // CSRF-токен
    if (data._csrf !== undefined) {
        // Обновление документа
        $('meta[name=csrf-token]').prop("content", data._csrf);
    }
}

function cart_response_success(data, status) {
    // Обработка ответов от удавшихся запросов

    cart_response(data, status);
}

function cart_response_error(data, status) {
    // Обработка ответов от неудавшихся запросов

    // Инициализвация
    data = data.responseJSON;

    cart_response(data, status);
}
